extension Array where Element: Hashable {
    /// Removes any duplicates from the array while maintaining the
    /// original order.
    public var uniques: Array {
        var buffer = Array()
        var added = Set<Element>()
        for elem in self {
            if !added.contains(elem) {
                buffer.append(elem)
                added.insert(elem)
            }
        }
        return buffer
    }
}

extension Array {
    /// Allows for safe array lookup, returning nil of the index is
    /// out of bounds.
    public subscript(safe index: Int) -> Element? {
        guard index >= 0, index < endIndex else {
            return nil
        }

        return self[index]
    }
}
